/* gw: Guess the word letter by letter. */
/* Original date: 2023-12-04 */
/* Version: 1.10 */
/* Date: 2024-10-31 */

#include <ctype.h>
#include <errno.h>
#include <math.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

/* Return a random floating point number between 0 and 1. */
static double
drand (unsigned long int seed)
{
  static bool first = true;
  static double r;
  if (first == true)
    {
      unsigned int srand_seed = seed;
      if (seed == 0)
	{
	  srand_seed = time (NULL) + clock ();
	}
      srand (srand_seed);
      first = false;
      r = rand () / ((double) RAND_MAX + 1);
    }
  r = 100 * log (r);
  r = fabs (r - trunc (r));
  return r;
}

/* Read a line from a file. */
static int
get_line (char **linep, size_t *sizep, FILE * const fp)
{
  char *line = *linep;
  size_t size = *sizep;
  int c;
  bool skip = false;
  size_t k = 0;
  if (line == NULL)
    {
      size = 1;
      line = malloc (size);
      if (line == NULL)
	{
	  return EOF;
	}
      else
	{
	  *linep = line;
	  *sizep = size;
	}
    }
  while ((c = fgetc (fp)) != EOF && c != '\n')
    {
      if (skip == false)
	{
	  if (c == '#')
	    {
	      skip = true;
	    }
	  else
	    {
	      if (k == size - 1)
		{
		  size = size * 1.1 + 1;
		  line = realloc (*linep, size);
		  if (line == NULL)
		    {
		      *(*linep + k) = '\0';
		      return EOF;
		    }
		  else
		    {
		      *linep = line;
		      *sizep = size;
		    }
		}
	      line[k++] = c;
	    }
	}
    }
  line[k] = '\0';
  return c;
}

/* Read the files line by line and pick a line at random. */
static bool
read_files (const char *const program_name,
	    const unsigned long int file_count,
	    const char **file_list,
	    const size_t minlen,
	    const size_t maxlen,
	    const char *const alphabet,
	    size_t *const countp,
	    char **str1p, size_t *size1p, char **str2p, size_t *size2p)
{
  bool return_value = true;
  if ((maxlen == 0 || minlen <= maxlen) && *alphabet != '\0')
    {
      unsigned long int i;
      for (i = 0; i < file_count; i++)
	{
	  const char *const file_name = file_list[i];
	  FILE *fp = fopen (file_name, "r");
	  if (fp == NULL)
	    {
	      fprintf (stderr, "%s: ", program_name);
	      perror (file_name);
	      return_value = false;
	    }
	  else
	    {
	      char **strp[2] = { str1p, str2p };
	      size_t *sizep[2] = { size1p, size2p };
	      const size_t temp_start = 1;
	      size_t temp = temp_start;
	      size_t line_num = 1;
	      while (get_line (strp[temp], sizep[temp], fp) == '\n')
		{
		  char *const str = *strp[temp];
		  if (*str != '\0')
		    {
		      const size_t len = strlen (str);
		      if (len >= minlen && (maxlen == 0 || len <= maxlen))
			{
			  if (strspn (str, alphabet) == len)
			    {
			      ++*countp;
			      if (*countp * drand (0) <= 1)
				{
				  temp = 1 - temp;
				}
			    }
			}
		    }
		  line_num++;
		}
	      if (feof (fp) == 0)
		{
		  fprintf (stderr, "%s: %s:%zu: ", program_name, file_name,
			   line_num);
		  perror (NULL);
		  return_value = false;
		}
	      if (temp != temp_start)
		{
		  char *const str = *strp[0];
		  const size_t size = *sizep[0];
		  *strp[0] = *strp[1];
		  *sizep[0] = *sizep[1];
		  *strp[1] = str;
		  *sizep[1] = size;
		}
	      if (0 != fclose (fp))
		{
		  fprintf (stderr, "%s: ", program_name);
		  perror (file_name);
		  return_value = false;
		}
	    }
	}
    }
  return return_value;
}

/* Resize a string buffer. */
static bool
resize (char **strp, const size_t size)
{
  char *const str = realloc (*strp, size);
  if (str == NULL)
    {
      return false;
    }
  else
    {
      *strp = str;
      return true;
    }
}

/* Count the characters that are hidden and update the current state. */
static size_t
update_state (char *const state, const char *const word,
	      const char *const try)
{
  size_t count = 0;
  char *s = state;
  const char *w = word;
  do
    {
      if (strchr (try, *w) == NULL)
	{
	  count++;
	  *s++ = '?';
	}
      else
	{
	  *s++ = *w;
	}
    }
  while (*++w != '\0');
  *s = '\0';
  return count;
}

/* Count and print the characters that are available. */
static size_t
available (const char *const alphabet, char *const try)
{
  size_t count = 0;
  const char *s = alphabet;
  do
    {
      if (strchr (try, *s) == NULL)
	{
	  count++;
	  if (count != 1)
	    {
	      putchar (' ');
	    }
	  putchar (*s);
	}
    }
  while (*++s != '\0');
  return count;
}

/* Read the files line by line and pick the most common of the possible letters. */
static size_t
next_try (const char *const program_name,
	  const unsigned long int file_count,
	  const char **file_list,
	  const size_t word_len,
	  const size_t alphabet_len,
	  const char *const alphabet,
	  const char *const state, const char *const try, size_t frequency[])
{
  size_t return_value = alphabet_len;
  unsigned long int i;
  if (frequency != NULL)
    {
      char *str = NULL;
      size_t size = 0;
      size_t match_count = 0;
      for (i = 0; i < alphabet_len; i++)
	{
	  frequency[i] = 0;
	}
      for (i = 0; i < file_count; i++)
	{
	  const char *const file_name = file_list[i];
	  FILE *fp = fopen (file_name, "r");
	  if (fp == NULL)
	    {
	      fprintf (stderr, "%s: ", program_name);
	      perror (file_name);
	    }
	  else
	    {
	      size_t line_num = 1;
	      while (get_line (&str, &size, fp) == '\n')
		{
		  if (*str != '\0')
		    {
		      const size_t len = strlen (str);
		      if (len == word_len)
			{
			  if (strspn (str, alphabet) == len)
			    {
			      size_t j;
			      for (j = 0; j < len; j++)
				{
				  if (state[j] == '?')
				    {
				      if (strchr (try, str[j]) != NULL)
					{
					  break;
					}
				    }
				  else if (state[j] != str[j])
				    {
				      break;
				    }
				}
			      if (j == len)
				{
				  match_count++;
				  for (j = 0; j < len; j++)
				    {
				      if (state[j] == '?'
					  && strchr (str, str[j]) == str + j)
					{
					  const size_t k =
					    strchr (alphabet,
						    str[j]) - alphabet;
					  frequency[k]++;
					}
				    }
				}
			    }
			}
		    }
		  line_num++;
		}
	      if (feof (fp) == 0)
		{
		  fprintf (stderr, "%s: %s:%zu: ", program_name, file_name,
			   line_num);
		  perror (NULL);
		}
	      if (0 != fclose (fp))
		{
		  fprintf (stderr, "%s: ", program_name);
		  perror (file_name);
		}
	    }
	}
      if (str != NULL)
	{
	  free (str);
	  str = NULL;
	  size = 0;
	}
/* printf("%zu words\n", match_count); */
      if (match_count > 0)
	{
	  size_t max = 0;
	  size_t count = 0;
	  for (i = 0; i < alphabet_len; i++)
	    {
	      if (max < frequency[i])
		{
		  max = frequency[i];
		  count = 1;
		  return_value = i;
		}
	      else if (max == frequency[i])
		{
		  count++;
		}
	    }
	  if (max != 0)
	    {
	      if (count != 1)
		{
		  size_t target = count * drand (0);
		  count = 0;
		  for (i = 0; i < alphabet_len; i++)
		    {
		      if (frequency[i] == max)
			{
			  count++;
			  if (count > target)
			    {
			      return_value = i;
			      break;
			    }
			}
		    }
		}
	      return return_value;
	    }
	}
    }
  size_t count = 0;
  for (i = 0; i < alphabet_len; i++)
    {
      unsigned char c = alphabet[i];
      if (strchr (try, c) == NULL)
	{
	  count++;
	}
    }
  size_t target = count * drand (0);
  count = 0;
  for (i = 0; i < alphabet_len; i++)
    {
      unsigned char c = alphabet[i];
      if (strchr (try, c) == NULL)
	{
	  count++;
	  if (count > target)
	    {
	      return_value = i;
	      break;
	    }
	}
    }
  return return_value;
}

int
main (int argc, char *argv[])
{
  const char *const program_name = argv[0];
  int return_value = EXIT_SUCCESS;
  const char *const default_word_list = "/var/www/main/data/word_list.txt";
  const char alphabet[] = "abcdefghijklmnopqrstuvwxyz";
  size_t word_size = 0;
  char *word = NULL;
  size_t temp_size = 0;
  char *temp = NULL;
  char try[sizeof (alphabet)] = "";
  enum
  { comment, last_boolean_opt };
  enum
  { computer_max_excess_incorrect_tries, max_excess_incorrect_tries,
      computer_max_incorrect_tries, max_incorrect_tries, max_word_len,
      min_word_len, computer_max_incorrect_tries_in_a_row,
      max_incorrect_tries_in_a_row, seed, computer_max_tries, max_tries,
      last_unsigned_opt };
  bool boolean_opt[last_boolean_opt] = { false };
  unsigned long int unsigned_opt[last_unsigned_opt] = { 0 };
  bool end = false;
  unsigned long int file_count = 0;
  const char **file_list = NULL;
  size_t word_count = 0;
  if (argc > 1)
    {
      enum opt_type
      { info_opt_type, boolean_opt_type, unsigned_opt_type };
      enum
      { help, version, last_info_opt };
      struct opt
      {
	const char *const short_opt;
	const char *const long_opt;
	const char *const help;
	unsigned char len;
	const int type;
	const int item;
      };
      struct opt opt_array[] = {
	{"c", "comment", "Comment on each try.", 0, boolean_opt_type,
	 comment},
	{"E N", "computer-maximum-excess-incorrect-tries=N",
	 "Set the maximum number of excess incorrect tries for the computer to N.\nThis is the number of incorrect tries minus the number of correct tries.\nThe default is 0 for no maximum.",
	 0, unsigned_opt_type, computer_max_excess_incorrect_tries},
	{"e N", "maximum-excess-incorrect-tries=N",
	 "Set the maximum number of excess incorrect tries to N.\nThis is the number of incorrect tries minus the number of correct tries.\nThe default is 0 for no maximum.",
	 0, unsigned_opt_type, max_excess_incorrect_tries},
	{"I N", "computer-maximum-incorrect-tries=N",
	 "Set the maximum number of incorrect tries for the computer to N.\nThe default is 0 for no maximum.",
	 0, unsigned_opt_type, computer_max_incorrect_tries},
	{"i N", "maximum-incorrect-tries=N",
	 "Set the maximum number of incorrect tries to N.\nThe default is 0 for no maximum.",
	 0, unsigned_opt_type, max_incorrect_tries},
	{"L N", "maximum-word-length=N",
	 "Set the maximum word length to N. The default is 0 for no maximum.",
	 0, unsigned_opt_type, max_word_len},
	{"l N", "minimum-word-length=N",
	 "Set the minimum word length to N. The default is 0 for no minimum.",
	 0, unsigned_opt_type, min_word_len},
	{"R N", "computer-maximum-incorrect-tries-in-a-row=N",
	 "Set the maximum number of incorrect tries in a row for the computer to N.\nThe default is 0 for no maximum.",
	 0, unsigned_opt_type, computer_max_incorrect_tries_in_a_row},
	{"r N", "maximum-incorrect-tries-in-a-row=N",
	 "Set the maximum number of incorrect tries in a row to N.\nThe default is 0 for no maximum.",
	 0, unsigned_opt_type, max_incorrect_tries_in_a_row},
	{"s N", "seed=N",
	 "Set the seed to N. The default is 0 for an unknown seed.\nTo repeat a game, use the same seed and the same word lists.",
	 0, unsigned_opt_type, seed},
	{"T N", "computer-maximum-tries=N",
	 "Set the maximum number of tries for the computer to N.\nThe default is 0 for no maximum.",
	 0, unsigned_opt_type, computer_max_tries},
	{"t N", "maximum-tries=N",
	 "Set the maximum number of tries to N. The default is 0 for no maximum.",
	 0, unsigned_opt_type, max_tries},
	{"h", "help", "Output this help and exit.", 0, info_opt_type, help},
	{"V", "version", "Output version information and exit.", 0,
	 info_opt_type, version},
      };
      const size_t opt_count = sizeof (opt_array) / sizeof (opt_array[0]);
      size_t j;
      for (j = 0; j < opt_count; j++)
	{
	  const int type = opt_array[j].type;
	  const int item = opt_array[j].item;
	  if (type == info_opt_type && item < last_info_opt)
	    {
	    }
	  else if (type == boolean_opt_type && item < last_boolean_opt)
	    {
	    }
	  else if (type == unsigned_opt_type && item < last_unsigned_opt)
	    {
	    }
	  else
	    {
	      fprintf (stderr,
		       "%s: internal error: invalid option: %zu (type %d, item %d)\n",
		       program_name, j, type, item);
	      return_value = EXIT_FAILURE;
	      end = true;
	    }
	  if (opt_array[j].long_opt != NULL)
	    {
	      const char *const s = strchr (opt_array[j].long_opt, '=');
	      if (s != NULL)
		{
		  opt_array[j].len = s - opt_array[j].long_opt;
		}
	      else
		{
		  opt_array[j].len = strlen (opt_array[j].long_opt);
		}
	    }
	}
      unsigned int pass;
      for (pass = 0; pass < 2 && end == false; pass++)
	{
	  bool opt_end = false;
	  int i;
	  for (i = 1; i < argc && end == false; i++)
	    {
	      if (opt_end == false && argv[i][0] == '-' && argv[i][1] != '\0')
		{
		  if (argv[i][1] == '-' && argv[i][2] == '\0')
		    {
		      opt_end = true;
		    }
		  else
		    {
		      size_t opt_match_count = 0;
		      const char *s = argv[i] + 1;
		      const char *v = NULL;
		      bool more = false;
		      do
			{
			  size_t opt_match = 0;
			  if (more == false && *s == '-')
			    {
			      s++;
			      size_t j;
			      size_t len = strlen (s);
			      v = strchr (s, '=');
			      if (v != NULL)
				{
				  len = v - s;
				  v++;
				}
			      for (j = 0; j < opt_count; j++)
				{
				  if (opt_array[j].long_opt != NULL)
				    {
				      if (v == NULL
					  || opt_array[j].
					  long_opt[opt_array[j].len] == '=')
					{
					  if (len == opt_array[j].len
					      && strncmp (s,
							  opt_array[j].
							  long_opt, len) == 0)
					    {
					      opt_match_count = 1;
					      opt_match = j;
					      break;
					    }
					  else if (len < opt_array[j].len
						   && strncmp (s,
							       opt_array[j].
							       long_opt,
							       len) == 0)
					    {
					      if (opt_match_count == 0)
						{
						  opt_match_count = 1;
						  opt_match = j;
						}
					      else
						if ((opt_array[j].type !=
						     opt_array[opt_match].
						     type)
						    || (opt_array[j].item !=
							opt_array[opt_match].
							item))
						{
						  opt_match_count++;
						}
					    }
					}
				    }
				}
			      if (opt_match_count == 0)
				{
				  fprintf (stderr,
					   "%s: unrecognized option '%s'\n",
					   program_name, argv[i]);
				  return_value = EXIT_FAILURE;
				  end = true;
				}
			      else if (opt_match_count == 1)
				{
				  if (v == NULL)
				    {
				      if (opt_array[opt_match].
					  long_opt[opt_array[opt_match].
						   len] == '=')
					{
					  if (i + 1 < argc)
					    {
					      v = argv[++i];
					    }
					  else
					    {
					      fprintf (stderr,
						       "%s: option --%.*s must have an argument\n",
						       program_name,
						       opt_array[opt_match].
						       len,
						       opt_array[opt_match].
						       long_opt);
					      return_value = EXIT_FAILURE;
					      end = true;
					    }
					}
				    }
				}
			      else
				{
				  size_t j;
				  fprintf (stderr,
					   "%s: option '%s' is ambiguous: '--%.*s'",
					   program_name, argv[i],
					   opt_array[opt_match].len,
					   opt_array[opt_match].long_opt);
				  for (j = opt_match + 1; j < opt_count; j++)
				    {
				      if (opt_array[j].long_opt != NULL)
					{
					  if (v == NULL
					      || opt_array[j].
					      long_opt[opt_array[j].len] ==
					      '=')
					    {
					      if (len < opt_array[j].len
						  && strncmp (s,
							      opt_array[j].
							      long_opt,
							      len) == 0)
						{
						  fprintf (stderr,
							   ", '--%.*s'",
							   opt_array[j].len,
							   opt_array[j].
							   long_opt);
						}
					    }
					}
				    }
				  fputc ('\n', stderr);
				  return_value = EXIT_FAILURE;
				  end = true;
				}
			    }
			  if (end == false)
			    {
			      if (opt_match_count == 0)
				{
				  const unsigned c = *s++;
				  size_t j;
				  more = false;
				  for (j = 0; j < opt_count; j++)
				    {
				      if (opt_array[j].short_opt != NULL)
					{
					  if (c == opt_array[j].short_opt[0])
					    {
					      opt_match = j;
					      if (opt_array[opt_match].
						  short_opt[1] == ' ')
						{
						  if (*s != '\0')
						    {
						      v = s;
						      s = strchr (s, '\0');
						    }
						  else if (i + 1 < argc)
						    {
						      v = argv[++i];
						    }
						  else
						    {
						      fprintf (stderr,
							       "%s: option %c must have an argument\n",
							       program_name,
							       c);
						      return_value =
							EXIT_FAILURE;
						      end = true;
						    }
						}
					      else if (*s != '\0')
						{
						  more = true;
						}
					      break;
					    }
					}
				    }
				  if (j == opt_count)
				    {
				      if (isprint (c) != 0)
					{
					  fprintf (stderr,
						   "%s: invalid option '%c'\n",
						   program_name, c);
					}
				      else
					{
					  fprintf (stderr,
						   "%s: invalid option '\\x%02X'\n",
						   program_name, c);
					}
				      return_value = EXIT_FAILURE;
				      end = true;
				    }
				}
			      if (end == false)
				{
				  const int type = opt_array[opt_match].type;
				  const int item = opt_array[opt_match].item;
				  if (pass == 0)
				    {
				      if (type == info_opt_type)
					{
					  if (item == help)
					    {
					      size_t j;
					      printf
						("Usage: %s [OPTIONS] [FILES]\n",
						 program_name);
					      puts
						("Guess a word letter by letter. The word comes from one of the word list files.\nWord list files have one word on each line.");
					      printf
						("The default word list file is %s.\n\n",
						 default_word_list);
					      for (j = 0; j < opt_count; j++)
						{
						  if (opt_array[j].
						      short_opt != NULL
						      && opt_array[j].
						      long_opt != NULL)
						    {
						      printf ("-%s, --%s\n",
							      opt_array[j].
							      short_opt,
							      opt_array[j].
							      long_opt);
						    }
						  else if (opt_array[j].
							   short_opt != NULL)
						    {
						      printf ("-%s\n",
							      opt_array[j].
							      short_opt);
						    }
						  else if (opt_array[j].
							   long_opt != NULL)
						    {
						      printf ("--%s\n",
							      opt_array[j].
							      long_opt);
						    }
						  if (opt_array[j].help !=
						      NULL)
						    {
						      puts (opt_array[j].
							    help);
						    }
						}
					    }
					  else if (item == version)
					    {
					      puts ("gw 1.10");
					    }
					  end = true;
					}
				      else if (type == unsigned_opt_type)
					{
					  if (*v == '\0')
					    {
					      if (opt_match_count == 1)
						{
						  fprintf (stderr,
							   "%s: option --%.*s must not have a blank argument\n",
							   program_name,
							   opt_array
							   [opt_match].len,
							   opt_array
							   [opt_match].
							   long_opt);
						}
					      else
						{
						  fprintf (stderr,
							   "%s: option %c must not have a blank argument\n",
							   program_name,
							   opt_array
							   [opt_match].
							   short_opt[0]);
						}
					      return_value = EXIT_FAILURE;
					      end = true;
					    }
					  else
					    {
					      char *t = NULL;
					      errno = 0;
					      const unsigned long long int
						value = strtoul (v, &t, 0);
					      if (*t != '\0')
						{
						  fprintf (stderr,
							   "%s: invalid positive integer: \"%s\"\n",
							   program_name, v);
						  return_value = EXIT_FAILURE;
						  end = true;
						}
					      else if (errno != 0)
						{
						  fprintf (stderr, "%s: ",
							   program_name);
						  perror (v);
						  return_value = EXIT_FAILURE;
						  end = true;
						}
					      else
						{
						  unsigned_opt[item] = value;
						}
					    }
					}
				      else if (type == boolean_opt_type)
					{
					  boolean_opt[item] = true;
					}
				    }
				}
			    }
			}
		      while (more == true && end == false);
		    }
		}
	      else if (pass == 0)
		{
		  file_count++;
		}
	      else
		{
		  file_list[file_count++] = argv[i];
		}
	    }
	  if (pass == 0 && end == false)
	    {
	      if (file_count != 0)
		{
		  file_list = malloc (file_count * sizeof (*file_list));
		  if (file_list == NULL)
		    {
		      perror (program_name);
		      return_value = EXIT_FAILURE;
		      end = true;
		    }
		  else
		    {
		      file_count = 0;
		    }
		}
	    }
	}
    }
  if (end == false)
    {
      if (file_count == 0)
	{
	  file_count = 1;
	  file_list = malloc (file_count * sizeof (*file_list));
	  if (file_list == NULL)
	    {
	      perror (program_name);
	      return_value = EXIT_FAILURE;
	      end = true;
	      file_count = 0;
	    }
	  else
	    {
	      file_list[0] = default_word_list;
	    }
	}
      if (end == false)
	{
	  const size_t alphabet_len = sizeof (alphabet) - 1;
	  const size_t max = alphabet_len - 1;
	  if (unsigned_opt[max_excess_incorrect_tries] > max)
	    {
	      unsigned_opt[max_excess_incorrect_tries] = max;
	    }
	  if (unsigned_opt[max_incorrect_tries] > max)
	    {
	      unsigned_opt[max_incorrect_tries] = max;
	    }
	  if (unsigned_opt[max_incorrect_tries_in_a_row] > max)
	    {
	      unsigned_opt[max_incorrect_tries_in_a_row] = max;
	    }
	  if (unsigned_opt[max_tries] > alphabet_len)
	    {
	      unsigned_opt[max_tries] = alphabet_len;
	    }
	  drand (unsigned_opt[seed]);
	  if (read_files
	      (program_name, file_count, file_list,
	       unsigned_opt[min_word_len], unsigned_opt[max_word_len],
	       alphabet, &word_count, &word, &word_size, &temp,
	       &temp_size) == false)
	    {
	      return_value = EXIT_FAILURE;
	    }
	  else if (word == NULL || *word == '\0')
	    {
	      fprintf (stderr, "%s: cannot find a word to guess\n",
		       program_name);
	      return_value = EXIT_FAILURE;
	    }
	  else
	    {
	      const size_t len = strlen (word);
	      if (resize (&word, len + 1) == false
		  || resize (&temp, len + 1) == false)
		{
		  perror (program_name);
		  return_value = EXIT_FAILURE;
		}
	      else
		{
		  bool show_available = false;
		  bool show_state = true;
		  bool input = false;
		  size_t hidden_count = update_state (temp, word, try);
		  size_t incorrect_tries = 0;
		  size_t incorrect_tries_in_a_row = 0;
		  size_t correct_tries = 0;
		  size_t tries = 0;
		  bool computer = false;
		  char *t = try;
		  do
		    {
		      if (show_available == true)
			{
			  show_state = true;
			  if (input == false)
			    {
			      size_t incorrect_remaining = 0;
			      bool show = false;
			      if (unsigned_opt[max_incorrect_tries] != 0)
				{
				  incorrect_remaining =
				    unsigned_opt[max_incorrect_tries] -
				    incorrect_tries;
				  if (unsigned_opt[max_incorrect_tries] != 1)
				    {
				      show = true;
				    }
				}
			      if (unsigned_opt[max_excess_incorrect_tries] !=
				  0)
				{
				  const size_t remaining =
				    correct_tries +
				    unsigned_opt[max_excess_incorrect_tries] -
				    incorrect_tries;
				  if (incorrect_remaining == 0
				      || incorrect_remaining > remaining)
				    {
				      incorrect_remaining = remaining;
				    }
				  show = true;
				}
			      if (unsigned_opt[max_incorrect_tries_in_a_row]
				  != 0)
				{
				  const size_t remaining =
				    unsigned_opt[max_incorrect_tries_in_a_row]
				    - incorrect_tries_in_a_row;
				  if (incorrect_remaining == 0
				      || incorrect_remaining > remaining)
				    {
				      incorrect_remaining = remaining;
				    }
				  if (unsigned_opt
				      [max_incorrect_tries_in_a_row] != 1)
				    {
				      show = true;
				    }
				}
			      if (unsigned_opt[max_tries] != 0)
				{
				  const size_t remaining =
				    unsigned_opt[max_tries] - tries;
				  if (incorrect_remaining >= remaining)
				    {
				      show = false;
				    }
				  else if (incorrect_remaining != 0)
				    {
				      show = true;
				    }
				  if (remaining == 1)
				    {
				      if (unsigned_opt[max_tries] != 1)
					{
					  puts ("1 try remaining");
					}
				    }
				  else
				    {
				      printf ("%zu tries remaining\n",
					      remaining);
				    }
				}
			      if (show == true)
				{
				  if (incorrect_remaining == 1)
				    {
				      puts ("1 incorrect try remaining");
				    }
				  else
				    {
				      printf
					("%zu incorrect tries remaining\n",
					 incorrect_remaining);
				    }
				}
			    }
			  available (alphabet, try);
			  putchar ('\n');
			}
		      else
			{
			  show_available = true;
			}
		      if (show_state == true)
			{
			  show_state = false;
			  puts (temp);
			}
		      if (hidden_count == 0)
			{
			  if (boolean_opt[comment] == true)
			    {
			      if (computer == false)
				{
				  puts ("you guessed the word");
				}
			    }
			  end = true;
			}
		      else
			{
			  int c = '\0';
			  input = false;
			  while (hidden_count != 0 && end == false
				 && c != '\n')
			    {
			      c = getchar ();
			      if (c == EOF)
				{
				  if (input == true)
				    {
				      putchar ('\n');
				    }
				  end = true;
				}
			      else if (c != '\n')
				{
				  input = true;
				  if (strchr (alphabet, c) != NULL)
				    {
				      if (strchr (try, c) != NULL)
					{
					  if (boolean_opt[comment] == true)
					    {
					      printf ("%c already tried\n",
						      c);
					    }
					}
				      else
					{
					  show_available = false;
					  *t++ = c;
/* *t = '\0'; */
					  if (strchr (word, c) == NULL)
					    {
					      incorrect_tries++;
					      incorrect_tries_in_a_row++;
					      if ((unsigned_opt
						   [max_excess_incorrect_tries]
						   != 0
						   && incorrect_tries ==
						   correct_tries +
						   unsigned_opt
						   [max_excess_incorrect_tries])
						  || (incorrect_tries ==
						      unsigned_opt
						      [max_incorrect_tries])
						  || (incorrect_tries_in_a_row
						      ==
						      unsigned_opt
						      [max_incorrect_tries_in_a_row]))
						{
						  computer = true;
						}
					      if (boolean_opt[comment] ==
						  true)
						{
						  printf ("%c incorrect\n",
							  c);
						}
					    }
					  else
					    {
					      show_state = true;
					      incorrect_tries_in_a_row = 0;
					      correct_tries++;
					      hidden_count =
						update_state (temp, word,
							      try);
					      if (boolean_opt[comment] ==
						  true)
						{
						  printf ("%c correct\n", c);
						}
					    }
					  tries++;
					  if (hidden_count != 0)
					    {
					      if (tries ==
						  unsigned_opt[max_tries])
						{
						  computer = true;
						}
					    }
					}
				    }
				}
			      if (computer == true)
				{
				  size_t frequency[sizeof (alphabet) - 1] =
				    { 0 };
				  incorrect_tries = 0;
				  incorrect_tries_in_a_row = 0;
				  correct_tries = 0;
				  tries = 0;
				  while (hidden_count != 0 && computer == true
					 && end == false)
				    {
				      const size_t i =
					next_try (program_name, file_count,
						  file_list, len,
						  sizeof (alphabet) - 1,
						  alphabet, temp, try,
						  frequency);
				      if (i == sizeof (alphabet) - 1)
					{
					  return_value = EXIT_FAILURE;
					  end = true;
					}
				      else
					{
					  const unsigned char c = alphabet[i];
					  *t++ = c;
					  if (boolean_opt[comment] == false)
					    {
					      printf ("%c\n", c);
					    }
					  if (strchr (word, c) == NULL)
					    {
					      incorrect_tries++;
					      incorrect_tries_in_a_row++;
					      if ((unsigned_opt
						   [computer_max_excess_incorrect_tries]
						   != 0
						   && incorrect_tries ==
						   correct_tries +
						   unsigned_opt
						   [computer_max_excess_incorrect_tries])
						  || (incorrect_tries ==
						      unsigned_opt
						      [computer_max_incorrect_tries])
						  || (incorrect_tries_in_a_row
						      ==
						      unsigned_opt
						      [computer_max_incorrect_tries_in_a_row]))
						{
						  computer = false;
						}
					      if (boolean_opt[comment] ==
						  true)
						{
						  printf ("%c incorrect\n",
							  c);
						}
					    }
					  else
					    {
					      show_state = true;
					      incorrect_tries_in_a_row = 0;
					      correct_tries++;
					      hidden_count =
						update_state (temp, word,
							      try);
					      if (boolean_opt[comment] ==
						  true)
						{
						  printf ("%c correct\n", c);
						}
					    }
					  tries++;
					  if (hidden_count != 0)
					    {
					      if (tries ==
						  unsigned_opt
						  [computer_max_tries])
						{
						  computer = false;
						}
					    }
					}
				    }
				  if (end == false)
				    {
				      incorrect_tries = 0;
				      incorrect_tries_in_a_row = 0;
				      correct_tries = 0;
				      tries = 0;
				    }
				}
			    }
			}
		    }
		  while (end == false);
		}
	    }
	}
    }
  if (word != NULL)
    {
      free (word);
      word = NULL;
    }
  if (temp != NULL)
    {
      free (temp);
      temp = NULL;
    }
  if (file_list != NULL)
    {
      free (file_list);
      file_list = NULL;
      file_count = 0;
    }
  if (0 != fclose (stdout))
    {
      perror (program_name);
      return_value = EXIT_FAILURE;
    }
  return return_value;
}
